use super::*;

pub type MemoryLayout<'a>    = Vec<Layout<'a>>;
pub type MemoryLayoutMut<'a> = Vec<LayoutMut<'a>>;

macro_rules! declare_memory_layout {
    ($($val:ident),*) => {
        /*
            Creates Layout(Mut)
            - add $($val),* to toggle mut
            - add $($val:camel),* inside paste::item to make NameMut or Name
        */
        paste::item! {
            pub struct [< DefaultLayout$($val:camel),* >] {
            }
            
            impl [< DefaultLayout$($val:camel),* >] {
                pub fn new(parent: &$($val),* CDDesc) -> Vec<[< Layout$($val:camel),* >]> {
                    let mut layout = Vec::new();
                    
                    layout.push([< Layout$($val:camel),* >]::SystemArea(&$($val),* parent.system_area));
                    layout.push([< Layout$($val:camel),* >]::PVD(&$($val),* parent.pvd));

                    [< add_dir_and_subdir $(_$val),* >](&mut layout, &$($val),* parent.root);
                    layout
                }
            }

            pub enum [< Layout$($val:camel),* >]<'a> {
                SystemArea(&'a $($val),* SystemArea),
                PVD(&'a $($val),* PrimaryVolumeDescriptor),
                Directory{name: &'a $($val),* DirectoryName, properties: &'a $($val),* Properties},
                File(&'a $($val),* File)
            }           

            fn [< add_dir_and_subdir $(_$val),* >]<'a>(layout: &mut Vec<[< Layout$($val:camel),* >]::<'a>>, dir: &'a $($val),* Directory) {
                layout.push([< Layout$($val:camel),* >]::Directory{name: &$($val),* dir.name, properties: &$($val),* dir.properties});
                for file in dir.files.[< iter$(_$val),* >]() {
                    layout.push([< Layout$($val:camel),* >]::File(file));
                }

                for dir in dir.dirs.[< iter$(_$val),* >]() {
                    [< add_dir_and_subdir $(_$val),* >](layout, dir);
                }
            }
        }
    };
}

declare_memory_layout!(mut);
declare_memory_layout!();